@@ -45,7 +45,7 @@ class Release:
         - license: (str) the license
     """
     name = 'pytango'
-    version_info = (9, 3, 0)
+    version_info = (9, 3, 4)
     version = '.'.join(map(str, version_info[:3]))
     release = ''.join(map(str, version_info[3:]))
     separator = '.' if 'dev' in release or 'post' in release else ''
@@ -59,7 +59,7 @@ class Release:
     license = 'LGPL'
     authors = {'Coutinho': ('Tiago Coutinho', 'coutinho@esrf.fr')}
     author_lines = "\n".join(["%s <%s>" % x for x in authors.values()])
-    url = 'http://github.com/tango-cs/pytango'
+    url = 'http://gitlab.com/tango-controls/pytango'
     download_url = 'http://pypi.python.org/pypi/pytango'
-    platform = ['Linux', 'Windows XP/Vista/7/8']
+    platform = ['Linux', 'Windows XP/Vista/7/8/10']
     keywords = ['Tango', 'CORBA', 'binding']
